#!/bin/sh
#
# Gentoo-based package build script

# find out where the build script is located
tdir=`echo "$0" | sed 's%[\\/][^\\/][^\\/]*$%%'`
test "x$tdir" = "x$0" && tdir=.
scriptdir=`cd $tdir; pwd`

# find src directory.
# If scriptdir ends in SPECS, then topdir is $scriptdir/..
# If scriptdir ends in CYGWIN-PATCHES, then topdir is $scriptdir/../..
# Otherwise, we assume that topdir = scriptdir
topdir1=`echo ${scriptdir} | sed 's%/SPECS$%%'`
topdir2=`echo ${scriptdir} | sed 's%/CYGWIN-PATCHES$%%'`
if [ "x$topdir1" != "x$scriptdir" ] ; then # SPECS
	T=`cd ${scriptdir}/..; pwd`
elif [ "x$topdir2" != "x$scriptdir" ] ; then # CYGWIN-PATCHES
		T=`cd ${scriptdir}/../..; pwd`
else
		T=`cd ${scriptdir}; pwd`
fi

tscriptname=`basename $0 .sh`
export PN=`echo $tscriptname | sed -e 's/\-[^\-]*\-[^\-]*$//'`
export PN1=perl-${PN}
export PN2=lib${PN}0
export PN3=lib${PN}-devel
export PN4=${PN}-doc
export V=`echo $tscriptname | sed -e "s/${PN}\-//" -e 's/\-[^\-]*$//'`
export R=`echo $tscriptname | sed -e "s/${PN}\-${V}\-//"`
export PV=${PN}-${V}
export PV1=${PN1}-${V}
export PV2=${PN2}-${V}
export PV3=${PN3}-${V}
export PV4=${PN4}-${V}
export PVR=${PV}-${R}
export PVR1=${PV1}-${R}
export PVR2=${PV2}-${R}
export PVR3=${PV3}-${R}
export PVR4=${PV4}-${R}

# Package maintainers: if the original source is not distributed as a
# (possibly compressed) tarball, set the value of ${src_orig_pkg_name},
# and redefine the unpack() helper function appropriately.
#
# determine correct decompression option and tarball filename
export src_orig_pkg_name=
if [ -e "${src_orig_pkg_name}" ] ; then
	export opt_decomp=? # Make sure tar punts if unpack() is not redefined
elif [ -e ${PV}.tar.bz2 ] ; then
	export opt_decomp=j
	export src_orig_pkg_name=${PV}.tar.bz2
elif [ -e ${PV}.tar.gz ] ; then
	export opt_decomp=z
	export src_orig_pkg_name=${PV}.tar.gz
elif [ -e ${PV}.tgz ] ; then
	export opt_decomp=z
	export src_orig_pkg_name=${PV}.tgz
elif [ -e ${PV}.tar ] ; then
	export opt_decomp=
	export src_orig_pkg_name=${PV}.tar
else
	echo Cannot find original package.
	exit 1
fi

export src_orig_pkg=${T}/${src_orig_pkg_name}

# determine correct names for generated files
export src_pkg_name=${PVR1}-src.tar.bz2
export src_patch_name=${PVR1}.patch
export bin_pkg1_name=${PVR1}.tar.bz2
export bin_pkg2_name=${PVR2}.tar.bz2
export bin_pkg3_name=${PVR3}.tar.bz2
export bin_pkg4_name=${PVR4}.tar.bz2
export src_pkg=${T}/${src_pkg_name}
export src_patch=${T}/${src_patch_name}
export bin_pkg1=${T}/${bin_pkg1_name}
export bin_pkg2=${T}/${bin_pkg2_name}
export bin_pkg3=${T}/${bin_pkg3_name}
export bin_pkg4=${T}/${bin_pkg4_name}

export S=${T}/${PV}
export B=${S}/.build
export D=${S}/.inst
export srcinstdir=${S}/.sinst
export checkfile=${T}/${PVR1}.check

if [ -z "$MY_CFLAGS" ] ; then
	MY_CFLAGS="-O2 -pipe"
fi
if [ -z "$MY_LDFLAGS" ] ; then
	MY_LDFLAGS=""
fi
if [ -z "$MAKEOPTS" ] ; then
	MAKEOPTS="-j2"
fi
if [ -z "$SIG" ] ; then
	SIG=1		# set to 1 to turn on signing by default
fi
if [ -z "$MULTIPKG" ] ; then
	MULTIPKG=0			# set to 1 to create multiple binary packages,
fi						# e.g. when shared libraries are built

export install_docs="\
	ABOUT-NLS \
	ANNOUNCE \
	ANNOUNCEMENTS \
	AUTHOR \
	AUTHORS \
	BUG-REPORTS \
	BUGS \
	Changes \
	CONTRIBUTORS \
	COPYING \
	COPYING-DOCS \
	COPYING.LIB \
	COPYRIGHT \
	CREDITS \
	CHANGELOG \
	ChangeLog \
	FAQ \
	GPL \
	HACKING \
	HOW-TO-CONTRIBUTE \
	INSTALL \
	KNOWNBUG \
	LEGAL \
	LICENCE \
	LICENSE \
	MAINTAINERS \
	NEWS \
	NLS \
	NOTES \
	PROGLIST \
	README \
	THANKS \
	TODO \
	WHATSNEW \
"

# helper function
# unpacks the original package source archive into ./${PV}/
# change this if the original package was not tarred
# or if it doesn't unpack to a correct directory
unpack() {
	tar xv${opt_decomp}f "$@"
}

dodir() {
	for d in "$@" ; do
		mkdir -p ${D}/${d}
	done
}

mkdirs() {
	cd ${T} && \
	rm -fr ${B} ${D} ${srcinstdir} && \
	mkdir -p ${B} ${D} ${srcinstdir}
}

prepsrc() {
	cd ${T} && \
	unpack ${src_orig_pkg} && \
	if [ -f ${src_patch} ] ; then \
		patch -Z -p0 < ${src_patch} ; \
	fi && \
	mkdirs && \
	lndir -silent ${S} ${B}
}

# Run this to re-autotool AFTER editing configure.{ac,in}/Makefile.am
autotool() {
	cd ${S} && \
	/usr/bin/autoreconf --install --force --verbose && \
	cd ${T} && \
	if [ -f ${PV}/INSTALL ] ; then \
		unpack ${src_orig_pkg} ${PV}/INSTALL ; \
	fi
}

cygconf() {
	cd ${B} && \
	perl Makefile.PL "$@" PREFIX=/usr INSTALLDIRS=vendor
}

reconf() {
	cd ${T} && \
	rm -fr ${B} && \
	mkdir -p ${B} && \
	cygconf
}

cygmake() {
	cd ${B} && \
	make ${MAKEOPTS} all manifypods "$@"
}

check() {
	cd ${B} && \
	if [ -f ${S}/Makefile.am ] ; then
		make check "$@" | tee ${checkfile} 2>&1
	else
		make test "$@" | tee ${checkfile} 2>&1
	fi
}

clean() {
	cd ${B} && make clean
}

cyginstall() {
	cd ${B} && \
	rm -fr ${D}/* && \
	make install DESTDIR=${D} "$@"

	dodir /usr/share/perl/cygwin-pods
	for f in `find ${D}/usr/lib/perl5 -name perllocal.pod` ; do
		sed -e "s:${D}::g" ${f} > ${D}/usr/share/perl/cygwin-pods/${PN}.pod ;
		rm -f ${f} ;
	done

	sed -i -e "s:${D}::g" `find ${D}/usr/lib/perl5 -name .packlist`
}

cygdoc() {
	echo "Installing documentation:"

	cd ${S} && \
	dodir /usr/share/doc/Cygwin && \
	dodir /usr/share/doc/${PV1} && \

	for f in ${install_docs} ; do
		if [ -f ${S}/${f} ] ; then
			echo "    ${f}" ;
			/bin/install -m 644 ${S}/${f} ${D}/usr/share/doc/${PV1} ;
		fi ; \
	done && \

	if [ -f ${S}/CYGWIN-PATCHES/README ] ; then
		/bin/install -m 644 ${S}/CYGWIN-PATCHES/README \
			${D}/usr/share/doc/Cygwin/${PV1}.README ;
	fi && \

	for f in ${PN1} ${PN2} ${PN3} ${PN4} ; do
		if [ -f ${S}/CYGWIN-PATCHES/${f}.README ] ; then
			/bin/install -m 644 ${S}/CYGWIN-PATCHES/${f}.README \
				${D}/usr/share/doc/Cygwin/${f}.README ;
		fi ;
	done
}

cygetc() {
	echo "Installing configuration files:"
	cd ${S} && \
	for d in postinstall preremove profile.d ; do
		if [ -f ${S}/CYGWIN-PATCHES/${d}.sh ] ; then \
			echo "    etc/${d}/${PN1}.sh" ; \
			dodir /etc/${d} ; \
			/bin/install -m 755 ${S}/CYGWIN-PATCHES/${d}.sh ${D}/etc/${d}/${PN1}.sh ; \
		fi
	done

	if [ -f ${S}/CYGWIN-PATCHES/profile.d.csh ] ; then \
		dodir /etc/profile.d ; \
		/bin/install -m 755 ${S}/CYGWIN-PATCHES/profile.d.csh \
			${D}/etc/profile.d/${PN1}.csh ; \
	fi

	dodir /etc/postinstall
	echo "#!/bin/sh
cat /usr/share/perl/cygwin-pods/${PN}.pod >> /usr/lib/perl5/5.8/cygwin/perllocal.pod" \
		>> ${D}/etc/postinstall/${PN1}.sh
}

prepallinfo() {
	cd ${D} && \
	for f in /usr/share/info/dir /usr/info/dir ; do \
		if [ -f ${D}${f} ] ; then \
			rm -f ${D}${f} ; \
		fi ; \
	done
	if [ -d ${D}/usr/share/info ] ; then \
		for i in `find usr/share/info -type f ! -name "*.gz"` ; do
			echo "gzipping info page: $i" ;
			gzip -q $i ;
		done
	fi
}

prepallman() {
	cd ${D} && \
	if [ -d ${D}/usr/share/man ] ; then \
		for m in `find usr/share/man -type f ! -name "*.gz"` ; do
			echo "gzipping man page: $m" ;
			gzip -q $m ;
		done ;
	fi
}

prepstrip() {
	echo "Stripping executables:"
	cd ${D} && \
	for e in `find . -name "*.dll" -or -name "*.exe"` ; do
		echo "    $e" ;
		strip $e 2>&1 ;
	done
}

list_files() {
	cd ${D} && \
	find . -name "*" ! -type d | sed 's%^\.%  %' ; \
	true
}

list_deps() {
	cd ${D} && \
	find ${D} -name "*.exe" -o -name "*.dll" | xargs -r cygcheck | \
		sed -e '/\.exe/d' -e 's,\\,/,g' | sort -bu | xargs -n1 cygpath -u \
		| xargs -r cygcheck -f | sort -bu | sed 's%^%  %' ; \
	true
}

cygpkg() {
	cd ${D} && \
	if [ "${MULTIPKG}" -eq 1 ] ; then \
		tar cvjf ${bin_pkg1} * \
			--exclude="usr/bin/*.dll" \
			--exclude="usr/bin/*-config" \
			--exclude="usr/include" \
			--exclude="usr/lib" \
			--exclude="usr/share/aclocal"\
			--exclude="usr/share/doc/Cygwin/${PV3}.README" ; \
		tar cvjf ${bin_pkg2} usr/bin/*.dll ; \
		tar cvjf ${bin_pkg3} usr/bin/*-config \
			usr/include/ \
			usr/lib/ \
			usr/share/aclocal ; \
			# usr/share/doc/Cygwin/${PV3}.README ; \
	else \
		tar cvjf ${bin_pkg1} * ; \
	fi
}

mkpatch() {
	cd ${S} && \
	find . -name "autom4te.cache" | xargs rm -rf ; \
	unpack ${src_orig_pkg} && \
	mv ${PV} ../${PV}-orig && \
	cd ${T} && \
	diff -urN -x '.build' -x '.inst' -x '.sinst' \
		${PV}-orig ${PV} > ${srcinstdir}/${src_patch_name} ; \
	rm -rf ${PV}-orig
}

cygspkg() {
	mkpatch && \
	if [ "${SIG}" -eq 1 ] ; then \
		name=${srcinstdir}/${src_patch_name} text="PATCH" sigfile ; \
	fi && \
	cp ${src_orig_pkg} ${srcinstdir}/${src_orig_pkg_name} && \
	if [ -e ${src_orig_pkg}.sig ] ; then \
		cp ${src_orig_pkg}.sig ${srcinstdir}/ ; \
	fi && \
	cp $0 ${srcinstdir}/`basename $0` && \
	name=${srcinstdir}/`basename $0` text="SCRIPT" sigfile && \
	cd ${srcinstdir} && \
	tar cvjf ${src_pkg} *
}

finish() {
	rm -rf ${S}
}

cleandir() {
	rm -fr ${src_orig_pkg} ${src_orig_pkg}.sig $0.sig ${src_patch} \
	${src_patch}.sig && \
	finish
}

sigfile() {
	if [ \( "${SIG}" -eq 1 \) -a \( -e $name \) -a \( \( ! -e $name.sig \) -o \( $name -nt $name.sig \) \) ] ; then \
		if [ -x /usr/bin/gpg ] ; then \
			echo "$text signature needs to be updated" ; \
			rm -f $name.sig ; \
			/usr/bin/gpg --detach-sign $name ; \
		else \
			echo "You need the gnupg package installed in order to make signatures." ; \
		fi ; \
	fi
}

checksig() {
	if [ -x /usr/bin/gpg ] ; then \
		if [ -e ${src_orig_pkg}.sig ] ; then \
			echo "ORIGINAL PACKAGE signature follows:" ; \
			/usr/bin/gpg --verify ${src_orig_pkg}.sig ${src_orig_pkg} ; \
		else \
			echo "ORIGINAL PACKAGE signature missing." ; \
		fi ; \
		if [ -e $0.sig ] ; then \
			echo "SCRIPT signature follows:" ; \
			/usr/bin/gpg --verify $0.sig $0 ; \
		else \
			echo "SCRIPT signature missing." ; \
		fi ; \
		if [ -e ${src_patch}.sig ] ; then \
			echo "PATCH signature follows:" ; \
			/usr/bin/gpg --verify ${src_patch}.sig ${src_patch} ; \
		else \
			echo "PATCH signature missing." ; \
		fi ; \
	else
		echo "You need the gnupg package installed in order to check signatures." ; \
	fi
}

while test -n "$1" ; do
	case $1 in
		prep|unpack)
			prepsrc ; STATUS=$? ;;
		mkdirs)
			mkdirs ; STATUS=$? ;;
		auto*)
			autotool ; STATUS=$? ;;
		conf*)
			cygconf ; STATUS=$? ;;
		reconf)
			reconf ; STATUS=$? ;;
		build|make)
			cygmake ; STATUS=$? ;;
		compile)
			cygconf && cygmake ; STATUS=$? ;;
		check|test)
			check ; STATUS=$? ;;
		clean)
			clean ; STATUS=$? ;;
		inst*)
			cyginstall && cygdoc && cygetc && \
			prepallinfo && prepallman && prepstrip ; STATUS=$? ;;
		list)
			list_files ; STATUS=$? ;;
		dep*)
			list_deps ; STATUS=$? ;;
		strip)
			prepstrip ; STATUS=$? ;;
		package|pkg)
			cygpkg ; STATUS=$? ;;
		mkpatch)
			mkpatch ; STATUS=$? ;;
		src-package|spkg)
			cygspkg ; STATUS=$? ;;
		finish)
			finish ; STATUS=$? ;;
		checksig)
			checksig ; STATUS=$? ;;
		first|ready)
			mkdirs && cygspkg && finish ; STATUS=$? ;;
		all)
			checksig && prepsrc && cygconf && cygmake && cyginstall && \
			cygdoc && cygetc && prepallinfo && prepallman && prepstrip && \
			cygpkg && cygspkg && finish ; STATUS=$? ;;
		*)
			echo "Error: bad arguments" ; exit 1 ;;
	esac
	( exit ${STATUS} ) || exit ${STATUS}
	shift
done
